// forge.ts
import { resolveFileSystem } from "@xmcl/system";
import { parse as parseToml } from "toml";
import { AnnotationVisitor, ClassReader, ClassVisitor, MethodVisitor, Opcodes } from "@xmcl/asm";
var ModAnnotationVisitor = class extends AnnotationVisitor {
  constructor(map) {
    super(Opcodes.ASM5);
    this.map = map;
  }
  visit(s, o) {
    if (s === "value") {
      this.map.modid = o;
    } else {
      this.map[s] = o;
    }
  }
};
var McVersionAnnotationVisitor = class extends AnnotationVisitor {
  constructor(map) {
    super(Opcodes.ASM5);
    this.map = map;
  }
  visit(s, o) {
    if (s === "value") {
      this.map(o);
    }
  }
};
var DummyModConstructorVisitor = class extends MethodVisitor {
  constructor(parent, api) {
    super(api);
    this.parent = parent;
  }
  stack = [];
  visitLdcInsn(value) {
    this.stack.push(value);
  }
  visitFieldInsn(opcode, owner, name, desc) {
    if (opcode === Opcodes.PUTFIELD) {
      const last = this.stack.pop();
      if (last) {
        if (name === "modId") {
          this.parent.guess.modid = last;
        } else if (name === "version") {
          this.parent.guess.version = last;
        } else if (name === "name") {
          this.parent.guess.name = last;
        } else if (name === "url") {
          this.parent.guess.url = last;
        } else if (name === "parent") {
          this.parent.guess.parent = last;
        } else if (name === "mcversion") {
          this.parent.guess.mcversion = last;
        }
      }
    }
  }
};
var ModClassVisitor = class extends ClassVisitor {
  constructor(result, guess, corePlugin) {
    super(Opcodes.ASM5);
    this.result = result;
    this.guess = guess;
    this.corePlugin = corePlugin;
  }
  fields = {};
  className = "";
  isDummyModContainer = false;
  isPluginClass = false;
  mcVersionInPlugin = "";
  pluginName = "";
  validateType(desc) {
    if (desc.indexOf("net/minecraftforge") !== -1) {
      this.result.usedForgePackage = true;
    }
    if (desc.indexOf("net/minecraft") !== -1) {
      this.result.usedMinecraftPackage = true;
    }
    if (desc.indexOf("cpw/mods/fml") !== -1) {
      this.result.usedLegacyFMLPackage = true;
    }
    if (desc.indexOf("net/minecraft/client") !== -1) {
      this.result.usedMinecraftClientPackage = true;
    }
  }
  visit(version, access, name, signature, superName, interfaces) {
    this.className = name;
    this.isPluginClass = name === this.corePlugin;
    if (superName === "net/minecraftforge/fml/common/DummyModContainer") {
      this.isDummyModContainer = true;
    }
    this.validateType(superName);
    for (const intef of interfaces) {
      this.validateType(intef);
      if (intef.indexOf("net/minecraftforge/fml/relauncher/IFMLLoadingPlugin") !== -1) {
        this.result.fmlPluginClassName = name;
      }
    }
  }
  visitMethod(access, name, desc, signature, exceptions) {
    if (this.isDummyModContainer && name === "<init>") {
      return new DummyModConstructorVisitor(this, Opcodes.ASM5);
    }
    this.validateType(desc);
    return null;
  }
  visitField(access, name, desc, signature, value) {
    this.fields[name] = value;
    return null;
  }
  visitAnnotation(desc, visible) {
    if (desc === "Lnet/minecraftforge/fml/common/Mod;" || desc === "Lcpw/mods/fml/common/Mod;") {
      const annotationData = {
        modid: "",
        name: "",
        version: "",
        dependencies: "",
        useMetadata: true,
        clientSideOnly: false,
        serverSideOnly: false,
        acceptedMinecraftVersions: "",
        acceptableRemoteVersions: "",
        acceptableSaveVersions: "",
        modLanguage: "java",
        modLanguageAdapter: "",
        value: ""
      };
      this.result.modAnnotations.push(annotationData);
      return new ModAnnotationVisitor(annotationData);
    } else if (desc === "Lnet/minecraftforge/fml/relauncher/IFMLLoadingPlugin$MCVersion;") {
      return new McVersionAnnotationVisitor((v) => {
        this.result.fmlPluginMcVersion = v;
      });
    }
    return null;
  }
  visitEnd() {
    if ((this.className === "Config" || this.className === "net/optifine/Config" || this.className === "notch/net/optifine/Config") && this.fields && this.fields.OF_NAME) {
      this.result.modAnnotations.push({
        modid: this.fields.OF_NAME,
        name: this.fields.OF_NAME,
        mcversion: this.fields.MC_VERSION,
        version: `${this.fields.OF_EDITION}_${this.fields.OF_RELEASE}`,
        description: "OptiFine is a Minecraft optimization mod. It allows Minecraft to run faster and look better with full support for HD textures and many configuration options.",
        authorList: ["sp614x"],
        url: "https://optifine.net",
        clientSideOnly: true,
        serverSideOnly: false,
        value: "",
        dependencies: "",
        useMetadata: false,
        acceptableRemoteVersions: "",
        acceptableSaveVersions: "",
        acceptedMinecraftVersions: `[${this.fields.MC_VERSION}]`,
        modLanguage: "java",
        modLanguageAdapter: ""
      });
    }
    for (const [k, v] of Object.entries(this.fields)) {
      switch (k.toUpperCase()) {
        case "MODID":
        case "MOD_ID":
          this.guess.modid = this.guess.modid || v;
          break;
        case "MODNAME":
        case "MOD_NAME":
          this.guess.name = this.guess.name || v;
          break;
        case "VERSION":
        case "MOD_VERSION":
          this.guess.version = this.guess.version || v;
          break;
        case "MCVERSION":
          this.guess.mcversion = this.guess.mcversion || v;
          break;
      }
    }
  }
};
async function readForgeModManifest(mod, manifestStore = {}) {
  const fs = await resolveFileSystem(mod);
  if (!await fs.existsFile("META-INF/MANIFEST.MF")) {
    return void 0;
  }
  const data = await fs.readFile("META-INF/MANIFEST.MF");
  const manifest = data.toString().split("\n").map((l) => l.trim()).filter((l) => l.length > 0).map((l) => l.split(":").map((s) => s.trim())).reduce((a, b) => ({ ...a, [b[0]]: b[1] }), {});
  Object.assign(manifestStore, manifest);
  const metadata = {
    modid: "",
    name: "",
    authors: [],
    version: "",
    description: "",
    url: ""
  };
  if (typeof manifest.TweakName === "string") {
    metadata.modid = manifest.TweakName;
    metadata.name = manifest.TweakName;
  }
  if (typeof manifest.TweakAuthor === "string") {
    metadata.authors = [manifest.TweakAuthor];
  }
  if (typeof manifest.TweakVersion === "string") {
    metadata.version = manifest.TweakVersion;
  }
  if (manifest.TweakMetaFile) {
    const file = manifest.TweakMetaFile;
    if (await fs.existsFile(`META-INF/${file}`)) {
      const metadataContent = await fs.readFile(`META-INF/${file}`, "utf-8").then((s) => s.replace(/^\uFEFF/, "")).then(JSON.parse);
      if (metadataContent.id) {
        metadata.modid = metadataContent.id;
      }
      if (metadataContent.name) {
        metadata.name = metadataContent.name;
      }
      if (metadataContent.version) {
        metadata.version = metadataContent.version;
      }
      if (metadataContent.authors) {
        metadata.authors = metadataContent.authors;
      }
      if (metadataContent.description) {
        metadata.description = metadataContent.description;
      }
      if (metadataContent.url) {
        metadata.url = metadataContent.url;
      }
    }
  }
  return metadata;
}
async function readForgeModToml(mod, manifest, fileName = "mods.toml") {
  const fs = await resolveFileSystem(mod);
  const existed = await fs.existsFile("META-INF/" + fileName);
  const all = [];
  if (existed) {
    const str = await fs.readFile("META-INF/" + fileName, "utf-8");
    const root = parseToml(str);
    if (root.mods instanceof Array) {
      for (const mod2 of root.mods) {
        const tomlMod = mod2;
        const modObject = {
          modid: tomlMod.modId ?? "",
          authors: tomlMod.authors ?? root.authors ?? "",
          // eslint-disable-next-line no-template-curly-in-string
          version: tomlMod.version === "${file.jarVersion}" && typeof (manifest == null ? void 0 : manifest["Implementation-Version"]) === "string" ? manifest == null ? void 0 : manifest["Implementation-Version"] : tomlMod.version,
          displayName: tomlMod.displayName ?? "",
          description: tomlMod.description ?? "",
          displayURL: tomlMod.displayURL ?? root.displayURL ?? "",
          updateJSONURL: tomlMod.updateJSONURL ?? root.updateJSONURL ?? "",
          provides: tomlMod.provides ?? [],
          dependencies: [],
          logoFile: tomlMod.logoFile ?? "",
          credits: tomlMod.credits ?? "",
          loaderVersion: root.loaderVersion ?? "",
          modLoader: root.modLoader ?? "",
          issueTrackerURL: root.issueTrackerURL ?? ""
        };
        all.push(modObject);
      }
    }
    if (typeof root.dependencies === "object") {
      for (const mod2 of all) {
        const dep = root.dependencies[mod2.modid];
        if (dep) {
          mod2.dependencies = dep;
        }
      }
    }
  }
  return all;
}
async function readForgeModAsm(mod, manifest = {}) {
  const fs = await resolveFileSystem(mod);
  let corePluginClass;
  if (manifest) {
    if (typeof manifest.FMLCorePlugin === "string") {
      const clazz = manifest.FMLCorePlugin.replace(/\./g, "/");
      if (await fs.existsFile(clazz) || await fs.existsFile(`/${clazz}`) || await fs.existsFile(`/${clazz}.class`) || await fs.existsFile(clazz + ".class")) {
        corePluginClass = clazz;
      }
    }
  }
  const result = {
    usedForgePackage: false,
    usedLegacyFMLPackage: false,
    usedMinecraftClientPackage: false,
    usedMinecraftPackage: false,
    modAnnotations: []
  };
  const guessing = {};
  await fs.walkFiles("/", async (f) => {
    if (!f.endsWith(".class")) {
      return;
    }
    const data = await fs.readFile(f);
    const visitor = new ModClassVisitor(result, guessing, corePluginClass);
    new ClassReader(data).accept(visitor);
  });
  if (result.modAnnotations.length === 0 && guessing.modid && (result.usedForgePackage || result.usedLegacyFMLPackage)) {
    result.modAnnotations.push({
      modid: guessing.modid ?? "",
      name: guessing.name ?? "",
      version: guessing.version ?? "",
      dependencies: guessing.dependencies ?? "",
      useMetadata: guessing.useMetadata ?? false,
      clientSideOnly: guessing.clientSideOnly ?? false,
      serverSideOnly: guessing.serverSideOnly ?? false,
      acceptedMinecraftVersions: guessing.acceptedMinecraftVersions ?? "",
      acceptableRemoteVersions: guessing.acceptableRemoteVersions ?? "",
      acceptableSaveVersions: guessing.acceptableSaveVersions ?? "",
      modLanguage: guessing.modLanguage ?? "java",
      modLanguageAdapter: guessing.modLanguageAdapter ?? "",
      value: guessing.value ?? ""
    });
  }
  return result;
}
async function readForgeModJson(mod) {
  const fs = await resolveFileSystem(mod);
  const all = [];
  function normalize(json) {
    const metadata = {
      modid: "",
      name: "",
      description: "",
      version: "",
      mcversion: "",
      url: "",
      updateUrl: "",
      updateJSON: "",
      authorList: [],
      credits: "",
      logoFile: "",
      screenshots: [],
      parent: "",
      useDependencyInformation: false,
      requiredMods: [],
      dependencies: [],
      dependants: []
    };
    metadata.modid = json.modid ?? metadata.modid;
    metadata.name = json.name ?? metadata.name;
    metadata.description = json.description ?? metadata.description;
    metadata.version = json.version ?? metadata.version;
    metadata.mcversion = json.mcversion ?? metadata.mcversion;
    metadata.url = json.url ?? metadata.url;
    metadata.updateUrl = json.updateUrl ?? metadata.updateUrl;
    metadata.updateJSON = json.updateJSON ?? metadata.updateJSON;
    metadata.authorList = json.authorList ?? metadata.authorList;
    metadata.credits = json.credits ?? metadata.credits;
    metadata.logoFile = json.logoFile ?? metadata.logoFile;
    metadata.screenshots = json.screenshots ?? metadata.screenshots;
    metadata.parent = json.parent ?? metadata.parent;
    metadata.useDependencyInformation = json.useDependencyInformation ?? metadata.useDependencyInformation;
    metadata.requiredMods = json.requiredMods ?? metadata.requiredMods;
    metadata.dependencies = json.dependencies ?? metadata.dependencies;
    metadata.dependants = json.dependants ?? metadata.dependants;
    return metadata;
  }
  function readJsonMetadata(json) {
    const modList = [];
    if (json instanceof Array) {
      modList.push(...json);
    } else if (json.modList instanceof Array) {
      modList.push(...json.modList);
    } else if (json.modid) {
      modList.push(json);
    }
    all.push(...modList.map(normalize));
  }
  if (await fs.existsFile("mcmod.info")) {
    try {
      const json = JSON.parse((await fs.readFile("mcmod.info", "utf-8")).replace(/^\uFEFF/, ""));
      readJsonMetadata(json);
    } catch (e) {
    }
  } else if (await fs.existsFile("cccmod.info")) {
    try {
      const text = (await fs.readFile("cccmod.info", "utf-8")).replace(/^\uFEFF/, "").replace(/\n\n/g, "\\n").replace(/\n/g, "");
      const json = JSON.parse(text);
      readJsonMetadata(json);
    } catch (e) {
    }
  } else if (await fs.existsFile("neimod.info")) {
    try {
      const text = (await fs.readFile("neimod.info", "utf-8")).replace(/^\uFEFF/, "").replace(/\n\n/g, "\\n").replace(/\n/g, "");
      const json = JSON.parse(text);
      readJsonMetadata(json);
    } catch (e) {
    }
  } else {
    const files = await fs.listFiles("./");
    const infoFile = files.find((f) => f.endsWith(".info"));
    if (infoFile) {
      try {
        const text = (await fs.readFile(infoFile, "utf-8")).replace(/^\uFEFF/, "").replace(/\n\n/g, "\\n").replace(/\n/g, "");
        const json = JSON.parse(text);
        readJsonMetadata(json);
      } catch (e) {
      }
    }
  }
  return all;
}
async function readForgeMod(mod) {
  const fs = await resolveFileSystem(mod);
  try {
    const jsons = await readForgeModJson(fs);
    const manifest = {};
    const manifestMetadata = await readForgeModManifest(fs, manifest);
    const tomls = await readForgeModToml(fs, manifest);
    const base = await readForgeModAsm(fs, manifest).catch(() => ({
      usedLegacyFMLPackage: false,
      usedForgePackage: false,
      usedMinecraftPackage: false,
      usedMinecraftClientPackage: false,
      modAnnotations: []
    }));
    if (jsons.length === 0 && (!manifestMetadata || !manifestMetadata.modid) && tomls.length === 0 && base.modAnnotations.length === 0) {
      throw new ForgeModParseFailedError(mod, base, manifest);
    }
    const result = {
      mcmodInfo: jsons,
      manifest,
      manifestMetadata: (manifestMetadata == null ? void 0 : manifestMetadata.modid) ? manifestMetadata : void 0,
      modsToml: tomls,
      ...base
    };
    return result;
  } finally {
    if (mod !== fs)
      fs.close();
  }
}
var ForgeModParseFailedError = class extends Error {
  constructor(mod, asm, manifest) {
    super("Cannot find the mod metadata in the mod!");
    this.mod = mod;
    this.asm = asm;
    this.manifest = manifest;
    this.name = "ForgeModParseFailedError";
  }
};

// forgeConfig.ts
var CorruptedForgeConfigError = class extends Error {
  constructor(reason, line) {
    super(`CorruptedForgeConfigError by ${reason}: ${line}`);
    this.reason = reason;
    this.line = line;
  }
  name = "CorruptedForgeConfigError";
};
var ForgeConfig;
((ForgeConfig2) => {
  function stringify(config) {
    let content = "# Configuration file\n\n\n";
    const propIndent = "    ";
    const arrIndent = "        ";
    Object.keys(config).forEach((cat) => {
      content += `${cat} {

`;
      config[cat].properties.forEach((prop) => {
        if (prop.comment) {
          const lines = prop.comment.split("\n");
          for (const l of lines) {
            content += `${propIndent}# ${l}
`;
          }
        }
        if (prop.value instanceof Array) {
          content += `${propIndent}${prop.type}:${prop.name} <
`;
          prop.value.forEach((v) => {
            content += `${arrIndent}${v}
`;
          });
          content += `${propIndent}>
`;
        } else {
          content += `${propIndent}${prop.type}:${prop.name}=${prop.value}
`;
        }
        content += "\n";
      });
      content += "}\n\n";
    });
    return content;
  }
  ForgeConfig2.stringify = stringify;
  function parse(body) {
    const lines = body.split("\n").map((s) => s.trim()).filter((s) => s.length !== 0);
    let category;
    let pendingCategory;
    const parseVal = (type, value) => {
      const map = {
        I: Number.parseInt,
        D: Number.parseFloat,
        S: (s) => s,
        B: (s) => s === "true"
      };
      const handler = map[type];
      return handler(value);
    };
    const config = {};
    let inlist = false;
    let comment;
    let last;
    const readProp = (type, line) => {
      line = line.substring(line.indexOf(":") + 1, line.length);
      const pair = line.split("=");
      if (pair.length === 0 || pair.length === 1) {
        let value;
        let name;
        if (line.endsWith(" <")) {
          value = [];
          name = line.substring(0, line.length - 2);
          inlist = true;
        }
        if (!category) {
          throw new CorruptedForgeConfigError("MissingCategory", line);
        }
        config[category].properties.push(last = { name, type, value, comment });
      } else {
        inlist = false;
        if (!category) {
          throw new CorruptedForgeConfigError("MissingCategory", line);
        }
        config[category].properties.push({ name: pair[0], value: parseVal(type, pair[1]), type, comment });
      }
      comment = void 0;
    };
    for (const line of lines) {
      if (inlist) {
        if (!last) {
          throw new CorruptedForgeConfigError("CorruptedList", line);
        }
        if (line === ">") {
          inlist = false;
        } else if (line.endsWith(" >")) {
          last.value.push(parseVal(last.type, line.substring(0, line.length - 2)));
          inlist = false;
        } else {
          last.value.push(parseVal(last.type, line));
        }
        continue;
      }
      switch (line.charAt(0)) {
        case "#":
          if (!comment) {
            comment = line.substring(1, line.length).trim();
          } else {
            comment = comment.concat("\n", line.substring(1, line.length).trim());
          }
          break;
        case "I":
        case "D":
        case "S":
        case "B":
          readProp(line.charAt(0), line);
          break;
        case "<":
          break;
        case "{":
          if (pendingCategory) {
            category = pendingCategory;
            config[category] = { comment, properties: [] };
            comment = void 0;
          } else {
            throw new CorruptedForgeConfigError("MissingCategory", line);
          }
          break;
        case "}":
          category = void 0;
          break;
        default:
          if (!category) {
            if (line.endsWith("{")) {
              category = line.substring(0, line.length - 1).trim();
              config[category] = { comment, properties: [] };
              comment = void 0;
            } else {
              pendingCategory = line;
            }
          } else {
            throw new CorruptedForgeConfigError("Duplicated", line);
          }
      }
    }
    return config;
  }
  ForgeConfig2.parse = parse;
})(ForgeConfig || (ForgeConfig = {}));

// liteloader.ts
import { resolveFileSystem as resolveFileSystem2 } from "@xmcl/system";
async function readLiteloaderMod(mod) {
  const fs = await resolveFileSystem2(mod);
  try {
    const text = await fs.readFile("litemod.json", "utf-8").then((s) => s.replace(/^\uFEFF/, "")).catch(() => void 0);
    if (!text) {
      throw Object.assign(new Error("Illegal input type! Expect a jar file contains litemod.json"), {
        mod,
        name: "IllegalInputType"
      });
    }
    const metadata = JSON.parse(text.trim(), (key, value) => key === "revision" ? Number.parseInt(value, 10) : value);
    if (!metadata.version) {
      metadata.version = `${metadata.mcversion}:${metadata.revision || 0}`;
    }
    return metadata;
  } finally {
    if (fs !== mod)
      fs.close();
  }
}

// fabric.ts
import { resolveFileSystem as resolveFileSystem3 } from "@xmcl/system";
async function readFabricMod(file) {
  const fs = await resolveFileSystem3(file);
  try {
    const content = await fs.readFile("fabric.mod.json", "utf-8");
    return JSON.parse(content.replace(/^\uFEFF/g, "").replace(/\n/g, ""));
  } finally {
    if (file !== fs)
      fs.close();
  }
}

// quilt.ts
import { resolveFileSystem as resolveFileSystem4 } from "@xmcl/system";
async function readQuiltMod(file) {
  const fs = await resolveFileSystem4(file);
  try {
    const content = await fs.readFile("quilt.mod.json", "utf-8");
    return JSON.parse(content.replace(/^\uFEFF/g, "").replace(/\n/g, ""));
  } finally {
    if (fs !== file)
      fs.close();
  }
}
export {
  CorruptedForgeConfigError,
  ForgeConfig,
  ForgeModParseFailedError,
  readFabricMod,
  readForgeMod,
  readForgeModAsm,
  readForgeModJson,
  readForgeModManifest,
  readForgeModToml,
  readLiteloaderMod,
  readQuiltMod
};
//# sourceMappingURL=index.mjs.map
